/**
 * @author Roberto Cornice <cornice@lemm.de> * 
 * @returns {yumpuController_backendcontrol}
 * 
 * This class controls the backend configuration by getting collections
 * and sections if a correct API-Token is given
 * 
 */
yumpuController_backendcontrol	= function()	{
	/**
	 * 
	 * @type yumpuController_backendcontrol
	 */
	var me = this;
	
	/**
	 * @type jQuery the API token field
	 */
	this.apiTokenField;
	
	/**
	 * @type jQuery the hidden API token field
	 */
	this.apiTokenHiddenField;
	
	/**
	 * @type jQuery the collection selector
	 */
	this.collectionSelector;
	
	/**
	 * @type jQuery the section selector
	 */
	this.sectionSelector;
	
	/**
	 * @type jQuery the collection field
	 */
	this.collectionField;
	
	/**
	 * @type jQuery the hidden collection field
	 */
	this.collectionHiddenField;
	
	/**
	 * @type jQuery the section field
	 */
	this.sectionField;
	
	/**
	 * @type jQuery the hidden section field
	 */
	this.sectionHiddenField;
	
	/**
	 * @type string active API token
	 */
	this.activeApiToken;
	
	/**
	 * Set each field as property
	 */
	this.findFields	= function()	{
		me.apiTokenField			= $('[data-formengine-input-name*="[settings.yumpu.apitoken]"]');
		me.apiTokenHiddenField		= $("input[name='" + $(me.apiTokenField).attr('data-formengine-input-name') + "'");
		me.collectionSelector		= $('[name*="[settings.yumpu.collectionids]"]');
		me.sectionSelector			= $('[name*="[settings.yumpu.sectionids]"]');
		me.collectionField			= $('[data-formengine-input-params*="[settings.yumpu.collectionid]"]');
		me.collectionHiddenField	= $('input[name="' + $(me.collectionField).attr('data-formengine-input-name') + '"]');
		me.sectionField				= $('[data-formengine-input-params*="[settings.yumpu.sectionid]"]');
		me.sectionHiddenField		= $('input[name="' + $(me.sectionField).attr('data-formengine-input-name') + '"]');
	};
	
	/**
	 * hides the Input fields
	 */
	this.hideFields	= function()	{
		$(me.collectionSelector).closest(".form-section").next(".form-section").hide();
		$(me.sectionSelector).closest(".form-section").next(".form-section").hide();
	};
	
	/**
	 * Binds keyup and paste on the API token field
	 */
	this.controlApiTokenField	= function()	{
		$(me.apiTokenField).on("keyup paste", function(event) {
			if (!(event.keyCode > 13 && event.keyCode < 41 && event.keyCode !== 32)) {
				me.checkApiToken(me.apiTokenField);
			}
		});
		me.checkApiToken(me.apiTokenHiddenField);
	};
	
	/**
	 * Checks the API token and loads all collection on valid result
	 */
	this.checkApiToken	= function(apiToken)	{
		$(me.collectionSelector).find("option").remove();
		$(me.sectionSelector).find("option").remove();
		$(me.apiTokenField).css("background", "#ffdada");
		if( $(apiToken).val().trim().length === 32 )	{
			$.ajax({
				type: "GET",
				beforeSend: function (request) {
					request.setRequestHeader("X-ACCESS-TOKEN", $(apiToken).val().trim());
				},
				url: "https://api.yumpu.com/2.0/user.json",
				success: function (msg) {
					if( msg.state === "success" )	{
						$(me.apiTokenField).css("background", "#daffda");
						me.activeApiToken	= $(apiToken).val().trim();
						me.getCollections();
					} else {
						$(me.apiTokenField).css("background", "#ffdada");
						console.log( msg );
					}
				},
				error	: function(msg)	{
					$(me.apiTokenField).css("background", "#ffdada");
					console.log(msg);
				}
			});
		}
	};
	
	/**
	 * Loads all collections and binds the collection selector on change to get sections
	 */
	this.getCollections	= function()	{
		$(me.collectionSelector).off("change");
		$.ajax({
			type: "GET",
			beforeSend: function (request) {
				request.setRequestHeader("X-ACCESS-TOKEN", me.activeApiToken);
			},
			url: "https://api.yumpu.com/2.0/collections.json?limit=100",
			error: function(msg)	{
				console.log(msg);
			},
			success: function (msg) {
				if( msg.state === "success" )	{
					$(me.collectionSelector).find("option").remove();
					$(me.collectionSelector).append('<option value="">Sammlung auswählen</option>');
					for( var i in msg.collections )	{
						if( typeof msg.collections[i].id !== "undefined" )	{
							$(me.collectionSelector).append('<option value="'+msg.collections[i].id+'">'+msg.collections[i].name+' ('+msg.collections[i].id+')</option>');
						}
					}
					if( $(me.collectionSelector).find("option[value='"+$(me.collectionHiddenField).val().trim()+"']").length !== 0 )	{
						$(me.collectionSelector).val($(me.collectionHiddenField).val().trim());
					}
					$(me.collectionSelector).off("change").on("change", function()	{
						me.getSections(true);
					});
					me.getSections(false);
				} 
				else {
//					console.log(msg);
				}
			}
		});
	};
	
	/**
	 * Gets all sections of defined collection
	 * @param {boolean} setField if true, sets the collection selector to the value of the hidden collection field
	 * @returns {undefined}
	 */
	this.getSections	= function(setField)	{
		if( setField )	{
			$(me.collectionField).val( $(me.collectionSelector).val() );
			$(me.collectionHiddenField).val( $(me.collectionSelector).val() );
		}
		var parameters	= {
			id	: $(me.collectionSelector).val().trim(),
			return_fields	: "sections"
		};
		$.ajax({
			type: "GET",
			beforeSend: function (request) {
				request.setRequestHeader("X-ACCESS-TOKEN", me.activeApiToken);
			},
			url: "https://api.yumpu.com/2.0/collection.json",
			data	: parameters,
			success: function (msg) {
				if( msg.state === "success" )	{
					$(me.sectionSelector).find("option").remove();
					$(me.sectionSelector).append('<option value="">Sammlung auswählen</option>');
					for( var i in msg.collection[0].sections )	{
						if( typeof msg.collection[0].sections[i].id !== "undefined" )	{
							$(me.sectionSelector).append('<option value="'+msg.collection[0].sections[i].id.split("_")[1]+'">'+msg.collection[0].sections[i].name+' ('+msg.collection[0].sections[i].id.split("_")[1]+')</option>');
						}
					}
					if( $(me.sectionSelector).find("option[value='"+$(me.sectionHiddenField).val().trim()+"']").length !== 0 )	{
						$(me.sectionSelector).val($(me.sectionHiddenField).val().trim());
					}
					$(me.sectionSelector).off("change").on("change", function()	{
						$(me.sectionHiddenField).val( $(me.sectionSelector).val() );
						$(me.sectionField).val( $(me.sectionSelector).val() );
					});
				}
			}
		});
	};
	
	/**
	 * starts the action (huh!)
	 */
	this.init	= function()	{
		me.findFields();
		me.hideFields();
		me.controlApiTokenField();
	};
	
	me.init();
};